package com.gic.store.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.store.dao.mapper.TabStoreFieldSelectMapper;
import com.gic.store.dto.StoreFieldSelectDTO;
import com.gic.store.entity.TabStoreFieldSelect;
import com.gic.store.service.StoreFieldSelectService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author guojx
 * @date 2019/7/1 2:48 PM
 */
@Service("storeFieldSelectService")
public class StoreFieldSelectServiceImpl implements StoreFieldSelectService{
    @Autowired
    private TabStoreFieldSelectMapper tabStoreFieldSelectMapper;

    @Override
    public int save(StoreFieldSelectDTO storeFieldSelectDTO) {
        return tabStoreFieldSelectMapper.insert(EntityUtil.changeEntityNew(TabStoreFieldSelect.class, storeFieldSelectDTO));
    }

    @Override
    public TabStoreFieldSelect getNextSort(int storeFieldId, double sort) {
        return tabStoreFieldSelectMapper.selectNextByStoreFieldIdAndSort(storeFieldId, sort);
    }

    @Override
    public TabStoreFieldSelect getPrevSort(int storeFieldId, double sort) {
        return tabStoreFieldSelectMapper.selectPrevByStoreFieldIdAndSort(storeFieldId, sort);
    }

    @Override
    public TabStoreFieldSelect getStoreFieldSelectById(int storeFieldSelectId) {
        return tabStoreFieldSelectMapper.selectByPrimaryKey(storeFieldSelectId);
    }

    @Override
    public int updateSortById(int storeFieldSelectId, double sort) {
        TabStoreFieldSelect fieldSelect = new TabStoreFieldSelect();
        fieldSelect.setStoreFieldSelectId(storeFieldSelectId);
        fieldSelect.setSort(sort);
        tabStoreFieldSelectMapper.updateByPrimaryKeySelective(fieldSelect);
        return 1;
    }

    @Override
    public void setStoreFieldSelectSort(int storeFieldSelectId, int storeFieldId, int setValue) {
        if (setValue <= 1) {
            // 相当于置顶
            sortTopping(storeFieldId, storeFieldSelectId);
        } else {
            int start = setValue - 2;
            int size = 2;

            List<TabStoreFieldSelect> list = tabStoreFieldSelectMapper.listTwoElementExceptId(storeFieldId, start, size, storeFieldSelectId);
            if (CollectionUtils.isEmpty(list)) {
                // 置底
                sortBottom(storeFieldId, storeFieldSelectId);
            } else {
                Double sort;
                if (list.size() == 1) {
                    // 置底
                    sort = list.get(0).getSort() + 1d;
                } else {
                    sort = (list.get(0).getSort() + list.get(1).getSort()) / 2d;
                }
                TabStoreFieldSelect fieldSelect = new TabStoreFieldSelect();
                fieldSelect.setStoreFieldSelectId(storeFieldSelectId);
                fieldSelect.setSort(sort);
                tabStoreFieldSelectMapper.updateByPrimaryKeySelective(fieldSelect);
            }
        }
    }

    @Override
    public int delete(int storeFieldSelectId) {
        return tabStoreFieldSelectMapper.deleteByPrimaryKey(storeFieldSelectId);
    }

    @Override
    public int edit(int storeFieldSelectId, String storeFieldSelectName, String storeFieldSelectCode) {
        TabStoreFieldSelect tabStoreFieldSelect = new TabStoreFieldSelect();
        tabStoreFieldSelect.setStoreFieldSelectId(storeFieldSelectId);
        tabStoreFieldSelect.setStoreFieldSelectName(storeFieldSelectName);
        tabStoreFieldSelect.setStoreFieldSelectCode(storeFieldSelectCode);
        return tabStoreFieldSelectMapper.updateByPrimaryKeySelective(tabStoreFieldSelect);
    }

    @Override
    public Page<TabStoreFieldSelect> listStoreFieldSelect(int storeFieldId, String search, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<TabStoreFieldSelect> list = tabStoreFieldSelectMapper.listStoreFieldSelect(storeFieldId, search);
        return (Page<TabStoreFieldSelect>) list;
    }


    private void sortTopping(int storeFieldId, int storeFieldSelectId) {
        Double sort = tabStoreFieldSelectMapper.getMinSort(storeFieldId);
        TabStoreFieldSelect fieldSelect = new TabStoreFieldSelect();
        fieldSelect.setStoreFieldSelectId(storeFieldSelectId);
        fieldSelect.setSort(sort - 1d);
        tabStoreFieldSelectMapper.updateByPrimaryKeySelective(fieldSelect);
    }

    private void sortBottom(int storeFieldId, int storeFieldSelectId) {
        Double sort = tabStoreFieldSelectMapper.getMaxSort(storeFieldId);
        TabStoreFieldSelect fieldSelect = new TabStoreFieldSelect();
        fieldSelect.setStoreFieldSelectId(storeFieldSelectId);
        fieldSelect.setSort(sort + 1d);
        tabStoreFieldSelectMapper.updateByPrimaryKeySelective(fieldSelect);
    }

}
