package com.gic.store.service.outer;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.entity.TabStoreField;
import com.gic.store.service.StoreFieldApiService;
import com.gic.store.service.StoreFieldService;
import com.gic.store.utils.ErrorCode;
import com.gic.store.utils.StoreFieldValidParamUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author guojx
 * @date 2019/6/28 1:40 PM
 */
@Service("storeFieldApiService")
public class StoreFieldApiServiceImpl implements StoreFieldApiService{
    @Autowired
    private StoreFieldService storeFieldService;

    @Override
    public ServiceResponse<Integer> save(StoreFieldDTO storeFieldDTO) {
        if (StringUtils.isBlank(storeFieldDTO.getStoreFieldName())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "属性名称参数值错误");
        }
        if (storeFieldDTO.getStoreFieldType() == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "字段类型参数值错误");
        }
        if (StringUtils.isBlank(storeFieldDTO.getStoreRegionId())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "所属域参数值错误");
        }
        if (storeFieldDTO.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "企业ID参数值错误");
        }
        if (StringUtils.isNotBlank(storeFieldDTO.getErpRelCode())) {
            if (storeFieldService.isRepeatCode(storeFieldDTO.getEnterpriseId(), null, storeFieldDTO.getStoreFieldCode())) {
                return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "Code不能重复");
            }
        }
        ServiceResponse validParamResult = StoreFieldValidParamUtil.validStoreFieldDetail(storeFieldDTO.getStoreFieldType(), storeFieldDTO.getStoreFieldDetail());
        if (!validParamResult.isSuccess()) {
            return validParamResult;
        }
        storeFieldDTO.setCreateTime(new Date());
        storeFieldDTO.setUpdateTime(new Date());
        storeFieldDTO.setStatus(1);
        storeFieldDTO.setSort(storeFieldService.getMinSort(storeFieldDTO.getEnterpriseId()) - 1);
        return ServiceResponse.success(storeFieldService.save(storeFieldDTO));
    }

    @Override
    public ServiceResponse<Integer> upOrDown(int storeFieldId, boolean isUp) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键错误，查询不到数据");
        }
        TabStoreField needUpdateSort;
        if (isUp) {
            needUpdateSort = storeFieldService.getPrevSort(storeField.getEnterpriseId(), storeField.getSort());
            if (needUpdateSort == null) {
                return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "已经是第一条，不能上移");
            }
        } else {
            needUpdateSort = storeFieldService.getNextSort(storeField.getEnterpriseId(), storeField.getSort());
            if (needUpdateSort == null) {
                return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "已经是最后一条，不能下移");
            }
        }
        storeFieldService.updateSortById(needUpdateSort.getStoreFieldId(), storeField.getSort());
        storeFieldService.updateSortById(storeField.getStoreFieldId(), needUpdateSort.getSort());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> setStoreFieldSort(int storeFieldId, int setValue) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键错误，查询不到数据");
        }
        storeFieldService.setStoreFieldSort(storeFieldId, storeField.getEnterpriseId(), setValue);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> delete(int storeFieldId) {
        return ServiceResponse.success(storeFieldService.delete(storeFieldId));
    }

    @Override
    public ServiceResponse<Integer> deleteAll(int enterpriseId) {
        return ServiceResponse.success(storeFieldService.deleteAll(enterpriseId));
    }

    @Override
    public ServiceResponse<Integer> deleteBatch(List<Integer> storeFieldIds) {
        return ServiceResponse.success(storeFieldService.deleteBatch(storeFieldIds));
    }

    @Override
    public ServiceResponse<Integer> edit(StoreFieldDTO storeFieldDTO) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldDTO.getStoreFieldId());
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键错误，查询不到数据");
        }
        if (StringUtils.isBlank(storeFieldDTO.getStoreFieldName())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "属性名称参数值错误");
        }
        if (storeFieldDTO.getStoreFieldType() == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "字段类型参数值错误");
        }
        if (StringUtils.isBlank(storeFieldDTO.getStoreRegionId())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "所属域参数值错误");
        }
        ServiceResponse validParamResult = StoreFieldValidParamUtil.validStoreFieldDetail(storeFieldDTO.getStoreFieldType(), storeFieldDTO.getStoreFieldDetail());
        if (!validParamResult.isSuccess()) {
            return validParamResult;
        }
        storeFieldDTO.setCreateTime(storeField.getCreateTime());
        storeFieldDTO.setUpdateTime(new Date());
        storeFieldDTO.setErpRelCode(storeField.getErpRelCode());
        return ServiceResponse.success(storeFieldService.edit(storeFieldDTO));
    }

    @Override
    public ServiceResponse<Page<StoreFieldDTO>> listStoreField(int enterpriseId, String search, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page storeFieldList = storeFieldService.listStoreField(enterpriseId, search, pageNum, pageSize);
        Page<StoreFieldDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeFieldList, StoreFieldDTO.class);
        List<StoreFieldDTO> selectDTOList = page.getResult();
        int index = (pageNum - 1) * pageSize + 1;
        for (int i = 0, length = selectDTOList.size(); i < length; i++) {
            selectDTOList.get(i).setSeq(index + i);
        }
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<StoreFieldDTO> getStoreField(int storeFieldId) {
        return ServiceResponse.success(EntityUtil.changeEntityNew(StoreFieldDTO.class, storeFieldService.getStoreFieldById(storeFieldId)));
    }

    @Override
    public ServiceResponse<Integer> sortBottom(int storeFieldId) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键错误，查询不到数据");
        }
        storeFieldService.sortBottom(storeField.getEnterpriseId(), storeFieldId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> sortTopping(int storeFieldId) {
        TabStoreField storeField = storeFieldService.getStoreFieldById(storeFieldId);
        if (storeField == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键错误，查询不到数据");
        }
        storeFieldService.sortTopping(storeField.getEnterpriseId(), storeFieldId);
        return ServiceResponse.success();
    }
}
