package com.gic.store.web.controller;

import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dto.StoreGroupDTO;
import com.gic.store.service.StoreGroupApiService;
import com.gic.store.web.qo.storegroup.StoreGroupQO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 门店分组
 * @author guojx
 * @date 2019/6/26 11:03 AM
 */
@RestController
@RequestMapping("/store-group")
public class StoreGroupController {

    private static final Logger logger = LoggerFactory.getLogger(StoreGroupController.class);

    @Autowired
    private StoreGroupApiService storeGroupApiService;

    @RequestMapping("/list")
    public RestResponse listStoreGroup(StoreGroupQO storeGroupQO) {
        List<StoreGroupDTO> list = storeGroupApiService.listStoreGroup(transferQoToDTO(storeGroupQO));
        return RestResponse.success(changeListToTree(StoreGroupConstant.ALL_STORE_LEVEL, list));
    }

    @RequestMapping("/save")
    public RestResponse save(@Validated({StoreGroupQO.SaveValidView.class}) StoreGroupQO storeGroupQO) {
        return RestResponse.success(storeGroupApiService.save(transferQoToDTO(storeGroupQO)));
    }

    @RequestMapping("/edit-store-group-name")
    public RestResponse editName(@Validated({StoreGroupQO.EditNameValidView.class, StoreGroupQO.RemoveValidView.class}) StoreGroupQO storeGroupQO) {
        return RestResponse.success(storeGroupApiService.updateStoreGroupName(transferQoToDTO(storeGroupQO)));
    }

    @RequestMapping("/edit-parent-store-group-id")
    public RestResponse edit(@Validated({StoreGroupQO.EditParentIdValidView.class, StoreGroupQO.RemoveValidView.class}) StoreGroupQO storeGroupQO) {
        return RestResponse.success(storeGroupApiService.update(transferQoToDTO(storeGroupQO)));
    }

    @RequestMapping("/sort")
    public RestResponse sort(@Validated({StoreGroupQO.SortValidView.class}) StoreGroupQO storeGroupQO) {
        return RestResponse.success(storeGroupApiService.sort(transferQoToDTO(storeGroupQO), storeGroupQO.isUp()));
    }

    @RequestMapping("/remove")
    public RestResponse remove(@Validated({StoreGroupQO.RemoveValidView.class}) StoreGroupQO storeGroupQO) {
        return RestResponse.success(storeGroupApiService.remove(storeGroupQO.getStoreGroupId()));
    }

    private List<Map<String, Object>> changeListToTree(int storeGroupId, List<StoreGroupDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (StoreGroupDTO t : list) {
                if (storeGroupId == t.getParentStoreGroupId()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("parentGroupId", t.getParentStoreGroupId());
                    data.put("storeGroupId", t.getStoreGroupId());
                    data.put("storeGroupName", t.getStoreGroupName());
                    data.put("sort", t.getSort());
                    data.put("isDefault", t.getIsDefault());
                    data.put("groupLevel", t.getGroupLevel());
                    List<Map<String, Object>> children = this.changeListToTree(t.getStoreGroupId(), list);
                    data.put("children", children);
                    result.add(data);
                }
            }
        }
        return result;
    }

    private StoreGroupDTO transferQoToDTO(StoreGroupQO storeGroupQO) {
        StoreGroupDTO storeGroupDTO = EntityUtil.changeEntityNew(StoreGroupDTO.class, storeGroupQO);
        storeGroupDTO.setEnterpriseId(StoreGroupConstant.TEST_ENTERPRISE_ID);
        return storeGroupDTO;
    }
}
